
%% LOAD PROBLEM DATA

clear all; close all; clc
import casadi.*

% define the truss structure
% have a look at load_data.m to get a rough understanding
D = load_data;                          

m = numel(D.A);                         % number of bars
n = size(D.Coord,2);                    % number of nodes

% calculate stiffness matrix
[KLMI, Le] = calculate_stiffness(D);

%% OPTIMIZE STRUCTURE
opti = casadi.Opti();

% define area variables
x = opti.variable(m);
u = opti.variable(length(D.Fext));

opti.minimize( D.Fext' * u  )

% form LMI using only the invertible part of K
Klmi = KLMI(:,:,1)*x(1);
for i = 2:size(KLMI,3)
    Klmi = Klmi + KLMI(:,:,i)*x(i);
end

opti.subject_to( x'*Le <= 1e5 );
opti.subject_to(  0 <= x <= 200 );
opti.subject_to( Klmi*u == D.Fext );
opti.solver('ipopt')

opti.set_initial(x, D.A)
sol = opti.solve();
xopt = sol.value(x);

disp(' ')
disp('Calculated diameters:')
disp( xopt )                 
%% CHECK AND PLOT RESUTS

disp(' ')
disp('Sum of cross-sectional areas:')
disp(sum(xopt.*Le))

% plot structure
plot_structure(D, xopt')